/*
 * Copyright (c) 2003-2006 Erez Zadok
 * Copyright (c) 2003-2006 Stony Brook University
 * Copyright (c) 2003-2006 The Research Foundation of State University of New York
 * Copyright (c) 2004-2006 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2006 Gopala Surya
 */

#include "tracereader.h"
#include "util.h"
#include <string.h>

struct key_struct anon_keys[ANON_MAX_FIELDS] = {
	{ 0, "uid", "" , NULL, NULL},
        { 0, "gid", "" , NULL, NULL},
        { 0, "pid", "" , NULL, NULL},
        { 0, "path", "" , NULL, NULL},
        { 0, "pname", "" , NULL, NULL},
	{ 0, "str", "" , NULL, NULL},
	{ 0, "timestamp", "", NULL, NULL}
};
int make_key(struct decrypt_params *params, char *strkey)
{
	int len = strlen(strkey);
	int ishex = 0;
	if (len < 2 || len % 2 != 0)
		return 0;
	ishex = (strkey[0] = '0' && (strkey[1] == 'x' || strkey[1] == 'X'));
	if (ishex)
		params->keylen = (len - 2)/2;
	else
		params->keylen = len;
	if (params->keylen > EVP_MAX_KEY_LENGTH)
		return 1;
	if (ishex) {
		int i;
		char *ptr = params->key;
		for (i = 2; i < len; i+= 2) {
			char *ch = hex2byte(strkey + i);
			if (!ch)
				return 1;
			*ptr = *ch;
			ptr++;
		}
	} else {
		memcpy(params->key, strkey, params->keylen);
	}
	fprintf(stderr, "key len = %d\n", params->keylen);
	return 0;
}
int hmac_make_key(struct cksum_params *params, char *strkey)
{
	int len = strlen(strkey);
	int ishex = 0;
	if (len < 2 || len % 2 != 0)
		return 0;
	ishex = (strkey[0] = '0' && (strkey[1] == 'x' || strkey[1] == 'X'));
	if (ishex)
		params->keylen = (len - 2)/2;
	else
		params->keylen = len;
	if (params->keylen > HMAC_MAX_KEY_LENGTH)
		return 1;
	if (ishex) {
		int i;
		char *ptr = params->key;
		for (i = 2; i < len; i+= 2) {
			char *ch = hex2byte(strkey + i);
			if (!ch)
				return 1;
			*ptr = *ch;
			ptr++;
		}
	} else {
		memcpy(params->key, strkey, params->keylen);
	}
	fprintf(stderr, "key len = %d\n", params->keylen);
	return 0;
}
char *get_str_arg_type(field_t arg)
{
	static char buf[20];
	int i;
	for (i = 0; arg_desc[i].str_type != NULL; i++) {
		if (arg_desc[i].arg_type == arg)
			return arg_desc[i].str_type;
	}
	snprintf(buf, 20, "%d", arg);
	return buf;
}

char *hex2byte(char *digits)
{
        static char ret = 0;
        int i;
        for (i = 0; i < 2; i++) {
		ret = ret << 4;
                if ((digits[i] >= 'a' && digits[i] <= 'f')) {
                        ret |= ((digits[i] - 'a' + 0xa) & 0xf);
                } else if ((digits[i] >= 'A' && digits[i] <= 'F')) {
                        ret |= ((digits[i] - 'A' + 0xA) & 0xf);
                } else if ((digits[i] >= '0' && digits[i] <= '9')) {
                        ret |= ((digits[i] - '0') & 0xf);
                } else {
			return NULL;
		}
        }
        return &ret;
}

char *byte2hex(char byte)
{
        static char ret[2];
	int i;
        for (i = 1; i >= 0; i--) {
		char val = byte & 0xf;
                if (val >= 0xa && val <= 0xf) {
                        ret[i] = 'a' + val - 0xa;
                } else
                        ret[i] = '0' + val;
		byte = byte >> 4;
        }
        return ret;
}


char *type_to_str_type(op_t type)
{
	int i;
	for (i = 0; msg_desc[i].str_type != NULL; i++) {
		if (type == msg_desc[i].msg_type)
			return msg_desc[i].str_type;
	}
	return "MSG_UNKNOWN";
}


